// Robot.cpp: implementation of the CRobot class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Std.h"
#include "Robot.h"
#include "OGLView.h"
#include <math.h>
#include "hat.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


/* Lister. */


#define uBody_li 0
#define laar_li 1
#define legg_li 2
#define fot_li 3
#define uArm_li 4
#define oArm_li 5
#define leftArm_li 6
#define rightArm_li 7
#define head_li 8
#define robot_li 9
#define lBody_li 10

#define antallLister 11
bool list_changed[antallLister];
int list[antallLister];


/* Robot egenskaper. */

int cyl_qual = 7;

float body_vx=-50.0f, body_vy=0.0f, body_vz=0.0f;
float robot_z = 0.0f, robot_x = 0.0f, robot_y = 0.0f;

float r_finger_krumming[] =   {45.0f, 43.0f, 43.0f, 37.0f, 0.0f};
float r_haandledd_vx = -24.0f, r_haandledd_vy = 0.0f, r_haandledd_vz=20.0f;
float r_arm_vx = -38.0f, r_arm_vy = 23.0f, r_arm_vz = 0.0f;
float r_overarm_vx = -64.0f, r_overarm_vy = -23.0f, r_overarm_vz = 0.0f;

float l_finger_krumming[] =  {45.0f, 43.0f, 43.0f, 37.0f, 0.0f};
float l_haandledd_vx = -24.0f, l_haandledd_vy = 0.0f, l_haandledd_vz=-20.0f;
float l_arm_vx = -38.0f, l_arm_vy = -23.0f, l_arm_vz = 0.0f;
float l_overarm_vx = -64.0f, l_overarm_vy = 23.0f, l_overarm_vz = 0.0f;
float overarm_size = 0.25f;
float underarm_size = 0.20f;
float overarm_rad = 0.02f;
float underarm_rad = 0.015f;
float hand_size[] = {0.08f, 0.07f, 0.02f, 0.0005f, 0.0173f};

float uBody_size[] = {0.20f, 0.04f, 0.5f};
float skulder_size[] = {0.35f, 0.03f};
float skulder_kule = 0.04f;
float head_rad = 0.14f;
float hals_rad = 0.04f, hals_len = 0.04f;
float hals_vx = 30.0f, hals_vy = 0.0f, hals_vz = 0.0f;

float lBody_len = 0.25f, lBody_rad = 0.04f;
float laar_rad = 0.03f, laar_len = 0.4f, knee_rad = 0.03f, legg_rad = 0.02f, legg_len = 0.4f;
float fot_x = 0.15f, fot_y=0.06, fot_z=0.02, fot_v = 45.0f, fot_rad[]={0.03, 0.0f, 0.02f};
float l_laar_vx = 0.0f, l_laar_vy = 0.0f, l_laar_vz = 0.0f;
float l_legg_vx = 0.0f, l_legg_vy = 0.0f, l_legg_vz = 0.0f;
float l_fot_vx = -90.0f, l_fot_vy = 0.0f, l_fot_vz = 0.0f;

float r_laar_vx = -0.0f, r_laar_vy = 0.0f, r_laar_vz = 0.0f;
float r_legg_vx = 0.0f, r_legg_vy = 0.0f, r_legg_vz = 0.0f;
float r_fot_vx = -90.0f, r_fot_vy = 0.0f, r_fot_vz = 0.0f;
/*float l_laar_vx = 0, l_laar_vy = 0, l_laar_vz = 0;
float l_legg_vx = 0, l_legg_vy = 0, l_legg_vz = 0;
float l_fot_vx = -90, l_fot_vy = 0, l_fot_vz = 0;

float r_laar_vx = 0, r_laar_vy = 0, r_laar_vz = 0;
float r_legg_vx = 0, r_legg_vy = 0, r_legg_vz = 0;
float r_fot_vx = -90, r_fot_vy = 0, r_fot_vz = 0;
*/
float nese_v = -100.0f, nese_rad = 0.01f, nese_len = 0.10f;
float eyebrow_v[] = {-65.0f, 30.0f, -85.0f}, eyebrow_rad = 0.01f, eyebrow_len = 0.07f;
float eye_v[] = {-80.0f, 15.0f}, eye_rad = 0.02f;

#define ROBOT_MATERIAL CHROME
#define HEAD_MATERIAL BRIGHT_WHITE
#define EYE_MATERIAL GOLD


GLUquadricObj *glpQ;




//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRobot::CRobot()
{
	int i;
	glpQ = gluNewQuadric();
	for(i = 0; i < antallLister; i++) {
		list_changed[i] = true;
		list[i] = 0;
	}

}

CRobot::~CRobot()
{

}

void CRobot::drawRobot()
{
	//glEnable(GL_CULL_FACE);
	CMaterial::SetMaterial(ROBOT_MATERIAL);
	glTranslatef(robot_x, robot_y, robot_z);
	glPushMatrix();
		glRotatef(body_vx, 1.0f, 0.0f, 0.0f);
		glRotatef(body_vy, 0.0f, 1.0f, 0.0f);
		glRotatef(body_vz, 0.0f, 0.0f, 1.0f);
		drawUpperBody();
	glPopMatrix();
	glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
	glPushMatrix();
		drawLowerBody();
	glPopMatrix();
}

void CRobot::drawUpperBody()
{
	CMaterial::SetMaterial(CHROME);
	if(liste(uBody_li)) {
		glPushMatrix();
		glTranslatef(uBody_size[0]/2.0f, 0.0f, 0.0f);
		drawCylinder(uBody_size[1]/2.0f, uBody_size[2]);
		glTranslatef(-uBody_size[0], 0.0f, 0.0f);
		drawCylinder(uBody_size[1]/2.0f, uBody_size[2]);
		glPopMatrix();
		glTranslatef(0.0f, 0.0f, uBody_size[2]);
		glEndList();
	}
	if(liste(head_li)) {
		glPushMatrix();
			drawHead();
		glPopMatrix();
		glEndList();
	}
	glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
	drawArmer();

}

void CRobot::drawL_Arm()
{
	//GLUquadricObj *glpQ;
	//glpQ=gluNewQuadric();
	if(liste(leftArm_li)){
		glPushMatrix();

		glRotatef(l_overarm_vx, 1.0f, 0.0f, 0.0f),
		glRotatef(l_overarm_vy, 0.0f, 1.0f, 0.0f);
		glRotatef(l_overarm_vz, 0.0f, 0.0f, 1.0f);
		drawCylinder(overarm_rad, overarm_size);

		glTranslatef(0.0f, 0.0f, overarm_size);
		drawKule(overarm_rad);
		glRotatef(l_arm_vx, 1.0f, 0.0f, 0.0f),
		glRotatef(l_arm_vy, 0.0f, 1.0f, 0.0f);
		glRotatef(l_arm_vz, 0.0f, 0.0f, 1.0f);
		drawCylinder(underarm_rad, underarm_size);

		glTranslatef(0.0f, 0.0f, underarm_size);
		drawKule(underarm_rad);
		glRotatef(l_haandledd_vx, 1.0f, 0.0f, 0.0f);
		glRotatef(l_haandledd_vy, 0.0f, 1.0f, 0.0f);
		glRotatef(l_haandledd_vz, 0.0f, 0.0f, 1.0f);

		drawL_Hand();
		glPopMatrix();
		glEndList();
	}
}

void CRobot::drawR_Hand()
{
	GLUquadricObj *glpQ;
	glpQ=gluNewQuadric();
	// Finger length relative to longest finger.
	float rel_finger_length[] = {0.6f, 0.8f, 1.0f, 0.85f, 1.0f};

	float width = hand_size[1], length = hand_size[0], height = hand_size[2], 
		finger_width = hand_size[4], finger_space = hand_size[3];

	glPushMatrix();
	glScalef(1.0f, height/length, 1.0f);
	drawClosedCylinder(width/2, length);
	//glTranslatef(-1.0f, -0.5f, 0.0f);
	//DrawBox();
	glPopMatrix();

	glPushMatrix();
	glTranslatef(-width/2+finger_width/2, 0.0f, length);
	for(int i = 0; i < 4; i++) {  // Fingers
		glPushMatrix();
		drawFinger(finger_width, length * rel_finger_length[i], r_finger_krumming[i]);
		glPopMatrix();
		glTranslatef(finger_width+finger_space, 0.0f, 0.0f);
	}

	// Thumb
	glTranslatef(0.0f, 0.0f, -length/2);
	glRotatef(-20.0f, 0.0f, 1.0f, 0.0f);
	glRotatef(50.0f, 1.0f, 0.0f, 0.0f);
	//glRotatef(-85.0f, 0.0f, 0.0f, 1.0f);
	//glRotatef(20.0f, 0.0f, 1.0f, 0.0f);
	drawFinger(finger_width*1.5f, length*rel_finger_length[4], r_finger_krumming[4]);


	glPopMatrix();

/*	glTranslatef(-1.0f, 0.0f, -2.5f);
	gluCylinder(glpQ, 0.15f, 0.15f, 2.5f, 20, 20);
	glTranslatef(0.3f, 0.0f, 0.0f);
	gluCylinder(glpQ, 0.15f, 0.15f, 2.5f, 20, 20);
	glTranslatef(0.3f, 0.0f, 0.0f);
	gluCylinder(glpQ, 0.15f, 0.15f, 2.5f, 20, 20);
	*/
}

void CRobot::drawR_Arm()
{
	//GLUquadricObj *glpQ;
	//glpQ=gluNewQuadric();
	if(liste(rightArm_li)) {
		glPushMatrix();

		glRotatef(r_overarm_vx, 1.0f, 0.0f, 0.0f),
		glRotatef(r_overarm_vy, 0.0f, 1.0f, 0.0f);
		glRotatef(r_overarm_vz, 0.0f, 0.0f, 1.0f);
		drawCylinder(overarm_rad, overarm_size);
		
		glTranslatef(0.0f, 0.0f, overarm_size);
		drawKule(overarm_rad);
		glRotatef(r_arm_vx, 1.0f, 0.0f, 0.0f),
		glRotatef(r_arm_vy, 0.0f, 1.0f, 0.0f);
		glRotatef(r_arm_vz, 0.0f, 0.0f, 1.0f);
		drawCylinder(underarm_rad, underarm_size);

		glTranslatef(0.0f, 0.0f, underarm_size);
		drawKule(underarm_rad);
		glRotatef(r_haandledd_vx, 1.0f, 0.0f, 0.0f);
		glRotatef(r_haandledd_vy, 0.0f, 1.0f, 0.0f);
		glRotatef(r_haandledd_vz, 0.0f, 0.0f, 1.0f);

		drawR_Hand();
		glPopMatrix();
		glEndList();
	}
}

void CRobot::drawL_Hand()
{
	// Finger length relative to longest finger.
	float rel_finger_length[] = {0.6f, 0.8f, 1.0f, 0.85f, 1.0f};

	float width = hand_size[1], length = hand_size[0], height = hand_size[2], 
		finger_width = hand_size[4], finger_space = hand_size[3];

	glPushMatrix();
	glScalef(1.0f, height/length, 1.0f);
	drawClosedCylinder(width/2, length);
	//glTranslatef(-1.0f, -0.5f, 0.0f);
	//DrawBox();
	glPopMatrix();

	glPushMatrix();
	glTranslatef(-width/2+finger_width/2, 0.0f, length);
	for(int i = 0; i < 4; i++) {  // Fingers
		glPushMatrix(); 
		drawFinger(finger_width, length * rel_finger_length[i], l_finger_krumming[i]);
		glPopMatrix();
		glTranslatef(finger_width+finger_space, 0.0f, 0.0f);
	}

	// Thumb
	glTranslatef(-width, 0.0f, -length/2);
	//glRotatef(5.0f, 0.0f, 0.0f, 1.0f);
	glRotatef(20.0f, 0.0f, 1.0f, 0.0f);
	glRotatef(50.0f, 1.0f, 0.0f, 0.0f);
	drawFinger(finger_width*1.5f, length*rel_finger_length[4], -l_finger_krumming[4]);

	glPopMatrix();

/*	glTranslatef(-1.0f, 0.0f, -2.5f);
	gluCylinder(glpQ, 0.15f, 0.15f, 2.5f, 20, 20);
	glTranslatef(0.3f, 0.0f, 0.0f);
	gluCylinder(glpQ, 0.15f, 0.15f, 2.5f, 20, 20);
	glTranslatef(0.3f, 0.0f, 0.0f);
	gluCylinder(glpQ, 0.15f, 0.15f, 2.5f, 20, 20);
	*/
}

void CRobot::drawFinger(float width, float length, float k)
{
	GLUquadricObj *glpQ;
	glpQ=gluNewQuadric();

	float length_left = length;

	float part_length[] = {length/2, length/3, length/4};

	drawKule(width/2);	
	for(int i = 0; i < 3; i++) {
		glRotatef(k, 1.0f, 0.0f, 0.0f);
		gluCylinder(glpQ, width/2, width/2, part_length[i], 5, 5);
		glTranslatef(0.0f, 0.0f, part_length[i]);
		drawKule(width/2);
	}

}

void CRobot::finger_bevegelse_1()
{

}

void CRobot::drawCylinder(float rad, float height)
{
	gluCylinder(glpQ, rad, rad, height, cyl_qual, cyl_qual);
}

void CRobot::drawSkuldre()
{
	drawCylinder(skulder_size[1], skulder_size[2]);
}

void CRobot::drawArmer()
{
	// Origo midt mellom skuldrene, z peker nedover.
	//drawCylinder(0.1f, 1.0f);
	glPushMatrix();
		glRotatef(90.0f, 1.0f, 0.0f, 0.0f);	
		glRotatef(90.0f, 0.0f, 1.0f, 0.0f);
		glTranslatef(0.0f, 0.0f, -skulder_size[0]/2);
		drawCylinder(skulder_size[1], skulder_size[0]);
		drawKule(skulder_kule);
		glRotatef(-90.0f, 0.0f, 1.0f, 0.0f);
		glRotatef(-90.0f, 1.0f, 0.0f, 0.0f);
		drawR_Arm();
	glPopMatrix();
	glTranslatef(skulder_size[0]/2, 0.0f, 0.0f);
	drawKule(skulder_kule);
	drawL_Arm();
}

void CRobot::drawHead()
{
	//glTranslatef(0.0f, 0.0f, sk);
	glRotatef(hals_vx, 1.0f, 0.0f, 0.0f);
	glRotatef(hals_vy, 0.0f, 1.0f, 0.0f);
	glRotatef(hals_vz, 0.0f, 0.0f, 1.0f);
	drawCylinder(hals_rad, hals_len);
	glTranslatef(0.0f, 0.0f, hals_len*0.9f+head_rad);
	CMaterial::SetMaterial(HEAD_MATERIAL);
	gluSphere(glpQ, head_rad, 15, 15);
	glPushMatrix();
		drawFace();
	glPopMatrix();
	glRotatef(hat.vx, 1.0f, 0.0f, 0.0f);
	glTranslatef(0.0f, 0.0f, hat.z);
	glTranslatef(0.0f, 0.0f, head_rad*1.4);
	CMaterial::SetMaterial(BLACK_RUBBER);
	hat.draw();//drawHat();
	//drawKule(head_rad);
	CMaterial::SetMaterial(ROBOT_MATERIAL);
}

void CRobot::drawKule(float rad)
{
	gluSphere(glpQ, rad, cyl_qual, cyl_qual);
}

void CRobot::drawLowerBody()
{
	// z peker nedover
	glPushMatrix();
		glRotatef(90.0f, 0.0f, 1.0f, 0.0f);
		glTranslatef(0.0f, 0.0f, -lBody_len/2);
		drawCylinder(lBody_rad, lBody_len);
		drawKule(lBody_rad);
		glRotatef(-90.0f, 0.0f, 1.0f, 0.0f);
		drawR_Leg();
	glPopMatrix();
	glTranslatef(lBody_len/2, 0.0f, 0.0f);
	drawKule(lBody_rad);
	drawL_Leg();
}

void CRobot::drawL_Leg()
{
	// z peker nedover
	glRotatef(l_laar_vx, 1.0f, 0.0f, 0.0f);
	glRotatef(l_laar_vy, 0.0f, 1.0f, 0.0f);
	glRotatef(l_laar_vz, 0.0f, 0.0f, 1.0f);
	if(liste(laar_li)) {
		drawCylinder(laar_rad, laar_len);
		glTranslatef(0.0f, 0.0f, laar_len);
		glEndList();
	}
	glRotatef(l_legg_vx, 1.0f, 0.0f, 0.0f);
	glRotatef(l_legg_vy, 0.0f, 1.0f, 0.0f);
	glRotatef(l_legg_vz, 0, 0, 1);
	if(liste(legg_li)) {
		drawKule(knee_rad);
		drawClosedCylinder(legg_rad, legg_len);
		glTranslatef(0.0f, 0.0f, legg_len);
		glEndList();
	}
	glRotatef(-l_laar_vx - l_legg_vx, 1.0f, 0.0f, 0.0f);
	glRotatef(l_fot_vx, 1.0f, 0.0f, 0.0f);
	glRotatef(l_fot_vy, 0.0f, 1.0f, 0.0f);
	glRotatef(l_fot_vz, 0.0f, 0.0f, 1.0f);
	glPushMatrix();
		glScalef(1.0f, fot_z/fot_y, 1.0f);
		drawClosedCylinder(fot_y, fot_x);
	glPopMatrix();
}	

void CRobot::drawClosedCylinder(float r, float h)
{
	drawCylinder(r, h);
	glPushMatrix();
		glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
		gluDisk(glpQ, 0.0f, r, cyl_qual, cyl_qual);
		glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
		glTranslatef(0.0f, 0.0f, h);
		gluDisk(glpQ, 0.0f, r, cyl_qual, cyl_qual);
	glPopMatrix();
}

void CRobot::drawR_Leg()
{
	// z peker nedover
	glRotatef(r_laar_vx, 1.0f, 0.0f, 0.0f);
	glRotatef(r_laar_vy, 0.0f, 1.0f, 0.0f);
	glRotatef(r_laar_vz, 0.0f, 0.0f, 1.0f);
	if(liste(laar_li)) {
		drawCylinder(laar_rad, laar_len);
		glTranslatef(0.0f, 0.0f, laar_len);
		glEndList();
	}
	glRotatef(r_legg_vx, 1.0f, 0.0f, 0.0f);
	glRotatef(r_legg_vy, 0.0f, 1.0f, 0.0f);
	glRotatef(r_legg_vz, 0.0f, 0.0f, 1.0f);
	if(liste(legg_li)) {
		drawKule(knee_rad);
		drawClosedCylinder(legg_rad, legg_len);
		glTranslatef(0.0f, 0.0f, legg_len);
		glEndList();
	}
	glRotatef(-r_laar_vx-r_legg_vx, 1.0f, 0.0f, 0.0f);
	glRotatef(r_fot_vx, 1.0f, 0.0f, 0.0f);
	glRotatef(r_fot_vy, 0.0f, 1.0f, 0.0f);
	glRotatef(r_fot_vz, 0.0f, 0.0f, 1.0f);
	glPushMatrix();
		glScalef(1.0f, fot_z/fot_y, 1.0f);
		drawClosedCylinder(fot_y, fot_x);
	glPopMatrix();	
}

void CRobot::moveL_Leg(int laar, int legg)
{
	l_laar_vx += laar;
	l_legg_vx += legg;
}

void CRobot::moveR_Leg(int laar, int legg)
{
	r_laar_vx += laar; 
	r_legg_vx += legg;
}


void CRobot::rot_body_x(int v)
{
	body_vx += v;
}

void CRobot::move_robot_z(float z)
{
	robot_z += z;
}

void CRobot::move_robot_x(float x)
{
	robot_x += x;
}

void CRobot::move_robot_y(float y)
{
	robot_y += y;
}

void CRobot::moveR_Arm(float vx1, float vy1, float vx2, float vy2)
{
	r_overarm_vx += vx1; r_overarm_vy += vy1;
	r_arm_vx += vx2; r_arm_vy += vy2;
	list_changed[rightArm_li] = true;
}

void CRobot::moveL_Arm(float vx1, float vy1, float vx2, float vy2)
{
	l_overarm_vx += vx1; l_overarm_vy += vy1;
	l_arm_vx += vx2; l_arm_vy += vy2;
	list_changed[leftArm_li] = true;
}

void CRobot::drawFace()
{
	int i;
	CMaterial::SetMaterial(BLACK_RUBBER);
	glPushMatrix();
		glRotatef(nese_v, 1.0f, 0.0f, 0.0f);
		glTranslatef(0.0f, 0.0f, head_rad);
		drawClosedCylinder2(nese_rad, nese_len, 2);
	glPopMatrix();
	for(i = -1; i <= 1; i+=2) {
		glPushMatrix();
			glRotatef(eyebrow_v[0], 1.0f, 0.0f, 0.0f);
			glRotatef(eyebrow_v[1]*i, 0.0f, 1.0f, 0.0f);
			glRotatef(eyebrow_v[2]*i, 0.0f, 0.0f, 1.0f);
			glTranslatef(0.0f, 0.0f, head_rad);
			glRotatef(90.0f, 1.0f, 0.0f, 0.0f);
			drawClosedCylinder2(eyebrow_rad, eyebrow_len, 2);
		glPopMatrix();
		glPushMatrix();
			CMaterial::SetMaterial(EYE_MATERIAL);
			glRotatef(eye_v[0], 1.0f, 0.0f, 0.0f);
			glRotatef(eye_v[1]*i, 0.0f, 1.0f, 0.0f);
			glTranslatef(0.0f, 0.0f, head_rad);
			//glRotatef(90.0f, 0.0f, 1.0f, 0.0f);
			gluDisk(glpQ, 0.0f, eye_rad, 10, 10);
			CMaterial::SetMaterial(BLACK_RUBBER);
		glPopMatrix();
	}
}

void CRobot::rotHead(float k)
{
	hals_vz += k;
	list_changed[head_li] = true;
}

void CRobot::rotEyebrow(float k)
{
	eyebrow_v[2] += k;
	list_changed[head_li] = true;
}


void CRobot::moveL_LegTo(float x, float y)
{

	l_laar_vx = x;//calc_laar_angle(x,  y);
	l_legg_vx = y;//calc_legg_angle(x, y);
	//l_fot_vx = +l_laar_vx-90.0f;// + (90.0f - (180.0f - (360.0f - l_legg_vx) - l_laar_vx));
	//if(y > 0)
	//	l_laar_vx = -l_laar_vx;	
}

/*
	float cost, cost2, t, t2, r = laar_len, r2 = legg_len;
	float d = r*r-r2*r2+x*x+y*y, a = 4*r2*r2*x*x, b = 4*a*r2*r2*x, c = a*a;
	float PI = 3.14159;
	cost2 = (- b + sqrt(b*b-4*a*c))/(2*a);
	cost = (r2*cost2+x)/r;
	t = acos(cost);
	t2 = acos(cost2)-t;
	l_laar_vx = (t*180.0f/PI);
	l_legg_vx = (t2*180.0f/PI);
}
/*
	float cost, cost2, t, t2, r = laar_len, r2 = legg_len;
	float tmp1, tmp2, tmp3;
	
	tmp1 = sqrt(1-2*x*x-2*y*y+2*r*r);
	tmp2 = 2*r2;
	cost = (- 1 + tmp1) / (tmp2);
	t = acos(cost);
	cost2 = (cost*r-x)/r2;
	t2 = -abs(acos(-cost2));
	l_laar_vx = (t*180.0f/PI)-90.0f;
	l_legg_vx = (t2*180.0f/PI)+l_laar_vx;
*/

void CRobot::rot_head_y(float k)
{
	hals_vy += k;
	list_changed[head_li] = true;
}

void CRobot::rot_head_x(float k)
{
	hals_vx += k;
	list_changed[head_li] = true;

}

void CRobot::rotR_fot(float k1, float k2)
{
	r_fot_vx += k1;
	r_fot_vz += k2;
}

void CRobot::rotR_Arm(float k)
{
	r_overarm_vz += k;
}

void CRobot::rotL_Arm(float k)
{
	l_overarm_vz += k;
}

void CRobot::rotR_hand_y(float k)
{
	r_haandledd_vy += k;	
}

void CRobot::rotR_hand_x(float k)
{
	r_haandledd_vx += k;
}


void CRobot::moveR_finger(int i, float k)
{
	r_finger_krumming[i] += k;
}

void CRobot::drawClosedCylinder2(float r, float h, int qual)
{
	drawCylinder(r, h);
	glPushMatrix();
		glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
		gluDisk(glpQ, 0.0f, r, cyl_qual, qual);
		glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
		glTranslatef(0.0f, 0.0f, h);
		gluDisk(glpQ, 0.0f, r, cyl_qual, qual);
	glPopMatrix();
}

void CRobot::moveR_LegTo(float x, float y)
{
	r_laar_vx = x;//calc_laar_angle(x, y);
	r_legg_vx = y; //calc_legg_angle(x, y);
}

bool CRobot::liste(int listeNum)
{
	if(list_changed[listeNum]) {
		glDeleteLists(list[listeNum], 1);
		list[listeNum] = glGenLists(1);
		glNewList(list[listeNum], GL_COMPILE_AND_EXECUTE);
		list_changed[listeNum] = false;
		return true;
	}
	else
		glCallList(list[listeNum]);
	return false;
}

void CRobot::liste_Slutt(int listeNum)
{

}

float CRobot::calc_laar_angle(float x, float y)
{
	float c, A, C, D, ra = laar_len, rb = legg_len;
	float PI = 3.14159;
	c = sqrt(x*x+y*y);
	
	//C = acos((rb*rb+ra*ra-c*c)/(2*ra*rb));
	A = acos((ra*ra+x*x+y*y-rb*rb)/(2*ra*c));
	D = acos(x/c);
	return 90.0f-(D+A)*180.0f/PI;

}

float CRobot::calc_legg_angle(float x, float y)
{
	float c, A, C, D, ra = laar_len, rb = legg_len;
	float PI = 3.14159;
	c = sqrt(x*x+y*y);
	
	C = acos((rb*rb+ra*ra-c*c)/(2*ra*rb));
	//A = acos((ra*ra+x*x+y*y-rb*rb)/(2*ra*c));
	//D = acos(x/c);
	return 180.0f-C*180.0f/PI;
}

void CRobot::drawHat()
{
	
/*	GLfloat ctrlpoints[4][3] = {
		{0.0f, -head_rad-0.06f, -head_rad*0.7f},
		{0.0f, -head_rad/2-0.03, -head_rad*0.7f},
		{0.0f, -head_rad, 0.0f},
		{0.0f, -head_rad/2, 0.0f}};
	glMap1f(GL_MAP1_VERTEX_3,0.0,1.0,3,4,&ctrlpoints[0][0]);
	glEnable(GL_MAP1_VERTEX_3);

	//for(int j = 0; j < 360; j+=5) {
		glBegin(GL_LINE_STRIP);
		for (int i=0;i<=29;i++) {
			//glNormal3f(1.0f, 0.0f, 0.0f);
			glEvalCoord1f((GLfloat) i/30.0);
			//glRotatef(5.0f, 0.0f, 0.0f, 1.0f);
			glEvalCoord1f((GLfloat) i/30.0);
			//glEvalCoord1f((GLfloat) (i+1)/30.0);
			glRotatef(-5.0f, 0.0f, 0.0f, 1.0f);
			//glEvalCoord1f((GLfloat) (i+1)/30.0);
		}
		glEnd();
		//glRotatef(5.0f, 0.0, 0.0f, 1.0f);
	//}*/
}

void CRobot::rotR_hand_z(float k)
{
	r_haandledd_vz += k;
}

void CRobot::rotL_hand_x(float k)
{
	l_haandledd_vx += k;
}

void CRobot::rotL_hand_y(float k)
{
	l_haandledd_vy += k;
}

void CRobot::rotL_hand_z(float k)
{
	l_haandledd_vz += k;
}

void CRobot::move_robot_z_to(float k)
{
	robot_z = k;
}

void CRobot::setR_Arm(float x1, float y1, float x2, float y2)
{
	r_overarm_vx = x1;
	r_overarm_vy = y1;
	r_arm_vx = x2;
	r_arm_vy = y2;
	list_changed[rightArm_li] = true;
}

void CRobot::setL_Arm(float x1, float y1, float x2, float y2)
{
	l_overarm_vx = x1;
	l_overarm_vy = y1;
	l_arm_vx = x2;
	l_arm_vy = y2;
	list_changed[leftArm_li] = true;
}

void CRobot::moveHat_z(float k)
{
	hat.z += k;
	list_changed[head_li] = true;
}

void CRobot::set_body_vx(float v)
{
	body_vx = v;
}

void CRobot::setHat_vx(float k)
{
	hat.vx = k;
}
