// lamp.cpp : implementation GRotObject class
//

//#include "stdafx.h"
#include "roter.h"
#include "Mater.h"
#include <math.h>
#include "mainfrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// Bezier resolution when editing
// # pnts pr curvesegment
#define BEZ_EDIT 50 // must be even

// steps in rotation
#define ROT_STEP 22

// the logical coordinaterange of an objectdescription
#define BASIC_SIZE 1000

// since OGL and WAPI want angles in diff format:
#define GtR	0.01745277777778f
#define PI  3.14159265359f

// to identify single controlpoints
#define HITSQUARE max(BASIC_SIZE/100,2)

/////////////////////////////////
// GRotObject
IMPLEMENT_DYNCREATE(GRotObject, CDialog)

// GRotObject construction/destruction

GRotObject::GRotObject(UINT nDialogID,CWnd* pParent/*=NULL*/)
: CDialog(nDialogID,pParent)
{	
	//{{AFX_DATA_INIT(GRotObject)
	//}}AFX_DATA_INIT


	// Start with a curve
	// NOTICE y and z swap
	// since this is a rotation object
	// and we will see xz-plan
	Con.RemoveAll(); // to make sure
	Bez.RemoveAll(); // to make sure

	P3Dn p;
	p.x=1; p.y=0; p.z=1; Con.Add(p);
	p.x=5; p.y=0; p.z=2; Con.Add(p);
	p.x=3; p.y=0; p.z=6; Con.Add(p);
	p.x=1; p.y=0; p.z=8; Con.Add(p);


	m_IsMoving=FALSE;
	m_MovePix=-1;

	hilited=-1;
}

GRotObject::GRotObject()
{
}

void GRotObject::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(GRotObject)
	//}}AFX_DATA_MAP

}


GRotObject::~GRotObject()
{
}

BEGIN_MESSAGE_MAP(GRotObject, CDialog)
	//{{AFX_MSG_MAP(GRotObject)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_BN_CLICKED(IDC_BUT_UPDATE, OnButtonUpdate)
	ON_BN_CLICKED(IDC_BUT_BPLUSS, OnButtonAddB)
	ON_BN_CLICKED(IDC_BUT_BMINUS, OnButtonRemoveB)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



///////////////////////////////////////////////
// interface to editor view


void GRotObject::GDraw()
{
	// draws a rotationobject based on
	// contour Cont, in xz-plane rotate around y-axes
	
	// Produce complete Bezierpoints
	MakeCompleteB();

	float x1b,y1b,z1b,x2b,y2b,z2b;	// basic strip,
	float x1,y1,z1,x2,y2,z2;		// running points
	float x1l,y1l,z1l,x2l,y2l,z2l;	// last strip

	glPushMatrix();
 	glEnable(GL_NORMALIZE);

	int count=Bez.GetSize();
	int pix=0;
	while(pix < count-1)
	{

		x1b=Bez[pix].x;		y1b=Bez[pix].y;		z1b=Bez[pix].z;
		x2b=Bez[pix+1].x;	y2b=Bez[pix+1].y;	z2b=Bez[pix+1].z;
		
		// the basc points as described in contour window
		// (x1b,y1b,z1b),(x2b,y2b,z2b)

		// run round with this strip
		float v=0.0f;
		float dv=2.0f*PI/(1.0f*ROT_STEP);
		// need last pnt to calculate normals
		// initialized as prev to first, same as last
		x1l=x1b*(float)cos(2.0f*PI-dv)-y1b*(float)sin(2.0f*PI-dv);
		y1l=x1b*(float)sin(2.0f*PI-dv)+y1b*(float)cos(2.0f*PI-dv);
		z1l=z1b;
		x2l=x2b*(float)cos(2.0f*PI-dv)-y2b*(float)sin(2.0f*PI-dv);
		y2l=x2b*(float)sin(2.0f*PI-dv)+y2b*(float)cos(2.0f*PI-dv);
		z2l=z2b;

		int lix=0;
		glBegin(GL_QUAD_STRIP);
		while(lix<=ROT_STEP)
		{
			x1=x1b*(float)cos(v)-y1b*(float)sin(v);
			y1=x1b*(float)sin(v)+y1b*(float)cos(v);
			z1=z1b;

			x2=x2b*(float)cos(v)-y2b*(float)sin(v);
			y2=x2b*(float)sin(v)+y2b*(float)cos(v);
			z2=z2b;

			// cross product (a2b3-a3b2)(a3b1-a1b3)(a1b2-a2b1)
			// normal based on (p1->p1l) X (p1->p2)
			glNormal3f(	-((y1l-y1)*(z2-z1)-(z1l-z1)*(y2-y1)),
						-((z1l-z1)*(x2-x1)-(x1l-x1)*(z2-z1)),
						-((x1l-x1)*(y2-y1)-(y1l-y1)*(x2-x1))	);
			//glNormal3f(x1,y1,0.0f);
			glVertex3f(x1,y1,z1);

			// normal based on (p2->p1) X (p2->p2l)
			glNormal3f(	((y2-y1)*(z2l-z2)-(z2-z1)*(y2l-y2)),
						((z2-z1)*(x2l-x2)-(x2-x1)*(z2l-z2)),
						((x2-x1)*(y2l-y2)-(y2-y1)*(x2l-x2))	);
			//glNormal3f(x2,y2,0.0f);
			glVertex3f(x2,y2,z2);

			// remember last
			x1l=x1; y1l=y1; z1l=z1;
			x2l=x2; y2l=y2; z2l=z2;

			lix++;
			v+=dv;
		}
		glEnd();
		
		if((pix+2)%BEZ_EDIT==0)
			pix++;
		pix++;
	}
	glPopMatrix();
}


////////////////////////////////////////////
// data convert

void GRotObject::MakeCompleteB()
{
	// produce complete Beziercontour Bez from 
	// controlpnts Con
	// controlpoints are long within BASIC_SIZE
	// Bezier will be float within 1.0f
	
	// NB!!! watch the yz-conversion below

	// prepare floatdescriptions
	Bez.SetSize(BEZ_EDIT,BEZ_EDIT);
	
	// and build from scratch
	int remaining=Con.GetSize();
	int pix=0;
	P3Dn p0,p1,p2,p3;
	P3Df pf;
	while(remaining>=4)
	{
		p0=Con[pix];
		p1=Con[pix+1];
		p2=Con[pix+2];
		p3=Con[pix+3];
		remaining-=4;
		pix+=4;
		// ok, the controlpoints are p0,p1,p2,p3

		// and the bezier curve
		float xf,yf,zf;
		float t1=0.0f;
		float t2=0.0f;
		float t3=0.0f;
		for(int bix=0;bix<BEZ_EDIT;bix++)
		{
			// can be speeded up with B-tables
			xf=	(1.0f-3*t1+3*t2-t3)*p0.x+
				(3*t1-6*t2+3*t3)*p1.x+
				(3*t2-3*t3)*p2.x+
				t3*p3.x;
			yf=	(1.0f-3*t1+3*t2-t3)*p0.y+
				(3*t1-6*t2+3*t3)*p1.y+
				(3*t2-3*t3)*p2.y+
				t3*p3.y;
			zf=	(1.0f-3*t1+3*t2-t3)*p0.z+
				(3*t1-6*t2+3*t3)*p1.z+
				(3*t2-3*t3)*p2.z+
				t3*p3.z;

			pf.x=xf/(1.0f*BASIC_SIZE);
			pf.y=yf/(1.0f*BASIC_SIZE);
			pf.z=zf/(1.0f*BASIC_SIZE);
			Bez.Add(pf);

			t1+=1.0f/(1.0f*BEZ_EDIT);
			t2=t1*t1;
			t3=t2*t1;
		}
	}
}



BOOL GRotObject::OnInitDialog() 
{
	CDialog::OnInitDialog();
	return TRUE;
}

void GRotObject::OnOK() 
{
	// since we update continously, we
	// do not have any unpacking of data
	CDialog::OnOK();
}


void GRotObject::OnPaint() 
{
	CDialog::OnPaint();
	CClientDC dc(this);
	// some drawing tools
	CPen BlueDotPen(PS_DOT,-1,RGB(0,0,255));
	CPen BluePen(PS_SOLID,-1,RGB(0,0,255));
	CPen BlackDotPen(PS_DOT,-1,RGB(0,0,0));
	CPen BlackPen(PS_SOLID,-1,RGB(0,0,0));
	CPen RedPen(PS_SOLID,-1,RGB(1,0,0));
	CPen *pOldPen;
	// map it
	Map(&dc);
	// show the unit area
	pOldPen=dc.SelectObject(&BlueDotPen);
	dc.MoveTo(-BASIC_SIZE/2,0);
	dc.LineTo(BASIC_SIZE/2,0);
	dc.LineTo(BASIC_SIZE/2,BASIC_SIZE);
	dc.LineTo(-BASIC_SIZE/2,BASIC_SIZE);
	dc.LineTo(-BASIC_SIZE/2,0);
	dc.MoveTo(0,0);			
	dc.LineTo(0,BASIC_SIZE);

	// draw controlpoints from Con
	// If # points are not a multiple of 4,
	// remaining pnts are ignored
	int remaining=Con.GetSize();
	int pix=0;
	CPoint p0,p1,p2,p3;
	while(remaining>=4)
	{
		p0.x=Con[pix].x;	p0.y=Con[pix].z;
		p1.x=Con[pix+1].x;	p1.y=Con[pix+1].z;
		p2.x=Con[pix+2].x;	p2.y=Con[pix+2].z;
		p3.x=Con[pix+3].x;	p3.y=Con[pix+3].z;
		remaining-=4;
		pix+=4;
		// ok, the controlpoints can be drawn as p0,p1,p2,p3

		// draw handles , p0-p1 and p2-p3
		dc.SelectObject(BlackDotPen);
		dc.MoveTo(p0.x,p0.y);dc.LineTo(p1.x,p1.y);
		dc.MoveTo(p2.x,p2.y);dc.LineTo(p3.x,p3.y);

		// draw pntsquares
		if((hilited>=pix)&&(hilited<=pix+3))
			dc.SelectObject(RedPen);
		else
			dc.SelectObject(BlackPen);
		dc.Rectangle(	p0.x-HITSQUARE,p0.y-HITSQUARE,
						p0.x+HITSQUARE,p0.y+HITSQUARE);
		dc.Rectangle(	p1.x-HITSQUARE,p1.y-HITSQUARE,
						p1.x+HITSQUARE,p1.y+HITSQUARE);
		dc.Rectangle(	p2.x-HITSQUARE,p2.y-HITSQUARE,
						p2.x+HITSQUARE,p2.y+HITSQUARE);
		dc.Rectangle(	p3.x-HITSQUARE,p3.y-HITSQUARE,
						p3.x+HITSQUARE,p3.y+HITSQUARE);

	}
	// and finally the beziercurve segments from Bez
	MakeCompleteB();
	for(int bix=0;bix< Bez.GetSize();bix++)
	{
		float fx,fy;
		fx=Bez[bix].x;
		fy=Bez[bix].z;
		int x,z;
		x=(int)(BASIC_SIZE*Bez[bix].x);
		z=(int)(BASIC_SIZE*Bez[bix].z);
		
		if(bix%BEZ_EDIT==0)
			dc.MoveTo(x,z);
		else
			dc.LineTo(x,z);
	}

	dc.SelectObject(pOldPen);
}

int GRotObject::GetHitPoint(long x,long y, long z)
{
	// check for hit, roundrobin
	// return index, or -1 if no hit
	// params in logical coordinates
	static int pix=0;
	int count=Con.GetSize();
	int ix=0;
	long px,py;
	CRect hR;
	hR.SetRect(	x-HITSQUARE,y-HITSQUARE,
				x+HITSQUARE,y+HITSQUARE);
	while(ix<count)
	{
		pix++;
		if(pix>=count)
			pix=0;

		px=Con[pix].x;
		py=Con[pix].z;
		
		if(hR.PtInRect(CPoint(px,py)))
			return pix;
		ix++;
	}
	return -1;
}


void GRotObject::Map(CDC *pDC)
{
	CRect R;
	GetClientRect(&R);
	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowOrg(0,0);
	pDC->SetWindowExt(2*BASIC_SIZE/3,4*BASIC_SIZE/3);
	pDC->SetViewportOrg(R.Width()/3,4*R.Height()/5);
	pDC->SetViewportExt(R.Width(),-R.Height());
}

void GRotObject::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CDialog::OnLButtonDown(nFlags, point);
	CClientDC dc(this);
	m_LastP=point;
	// move attempt ?
	Map(&dc);
	dc.DPtoLP(&point);
	m_MovePix=GetHitPoint(point.x,point.y,0);
	if(m_MovePix!=-1)
	{
		SetCapture();
		m_IsMoving=TRUE;
	}
	int oldhilite=hilited;
	SetHilite(m_MovePix);
	if(oldhilite!=hilited)
		RedrawWindow();
}


void GRotObject::OnMouseMove(UINT nFlags, CPoint point) 
{
	// maybe we are moving a point
	if((nFlags & MK_LBUTTON)&&(m_IsMoving))
	{
		// substantial move ?
		if((abs(point.x-m_LastP.x)>1)||(abs(point.y-m_LastP.y)>1))
		{
			// do it
			m_LastP=point;
			CClientDC dc(this);
			Map(&dc);
			dc.DPtoLP(&point);

			Con[m_MovePix].x=point.x;
			Con[m_MovePix].z=point.y;
			RedrawWindow();
		}
	}
	//CDialog::OnMouseMove(nFlags, point);
}


void GRotObject::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// terminate move ?
	if(m_IsMoving)
	{
		ReleaseCapture();
		m_MovePix=-1;
		Invalidate(NULL);
		m_IsMoving=FALSE;
	}
	CDialog::OnLButtonUp(nFlags, point);
}

void GRotObject::OnButtonUpdate()
{
	// refresh
	(pMain->GetActiveView())->RedrawWindow();
}


void GRotObject::SetHilite(int bix)
{
	// make a note of the first of 4 pnts
	// that is the bezierpnts of which bix is one
	hilited=bix;
}

void GRotObject::OnButtonAddB()
{
	// add a bezier segment
	P3Dn p;
	if(Con.GetSize()==0)
	{
		p.x=100; p.y=0; p.z=100; 
	}
	else
		p=Con[Con.GetSize()-1];

	Con.Add(p);
	p.x+=2*HITSQUARE; p.y=0; p.z+=2*HITSQUARE;
	Con.Add(p);
	p.x+=2*HITSQUARE; p.y=0; p.z+=2*HITSQUARE;
	Con.Add(p);
	p.x+=2*HITSQUARE; p.y=0; p.z+=2*HITSQUARE;
	Con.Add(p);
	RedrawWindow();
}

void GRotObject::OnButtonRemoveB()
{
	// remove hilited bezier segment
	if(Con.GetSize()==0)
		return;
	if(hilited==0)
		return;
	int startix=(hilited/4)*4;
	Con.RemoveAt(startix+3);
	Con.RemoveAt(startix+2);
	Con.RemoveAt(startix+1);
	Con.RemoveAt(startix);
	RedrawWindow();
}


